'use strict'

let express = require('express')
let bodyParser = require('body-parser')
let jsonfile = require('jsonfile')
let path = require('path')
let fs = require('fs')
let api = express()

const pathJsonFile = path.join(__dirname, '/json/')
const pathApi = '/api'
jsonfile.spaces = 2

const ADD = 'add'
const UPD = 'upd'
const DEL = 'del'

let nbEntities

api.use(function (req, res, next) {
  res.header('Access-Control-Allow-Origin', '*')
  res.header('Access-Control-Allow-Headers', 'Origin, X-Requested-With, Content-Type, Accept')
  next()
})
api.use(bodyParser.json())

function isDefined (value) { return value !== undefined }
function isObject (value) { return typeof value === 'object' }

function createFolderIsNotExist (pathFolder) {
  if (!fs.existsSync(path.join(__dirname, pathFolder))) {
    fs.mkdir(path.join(__dirname, pathFolder), function (err) {
      if (!err || (err && err.code === 'EEXIST')) {
        console.log('Successful creation for the', pathFolder, 'folder on path', __dirname)
      } else {
        console.log('Error on create folder', pathFolder, 'on path', __dirname, err)
      }
    })
  }
}

function targetLevelForAction (obj, levels, i, indexLanguage, action, value, newValue) {
  for (let key in obj) {
    if (key === levels[i]) {
      if (key === levels[i] && i === levels.length - 1) {
        if (action === ADD || action === UPD) {
          if (isObject(value)) {
            if (action === ADD) {
              obj[key][value.key] = value.value[indexLanguage]
            } else {
              if (value.originalKey === value.key) {
                obj[key][value.key] = value.value[indexLanguage]
              } else {
                delete obj[key][value.originalKey]
                obj[key][value.key] = value.value[indexLanguage]
              }
            }
          } else {
            if (action === ADD) {
              obj[key][value] = {}
            } else {
              let copyContent = obj[key][value]
              delete obj[key][value]
              obj[key][newValue] = copyContent
            }
          }
          return obj
        } else if (action === DEL) {
          if (isObject(value)) {
            delete obj[key][value.key]
          } else {
            delete obj[key][value]
          }
          return obj
        }
      }
      i++
      targetLevelForAction(obj[key], levels, i, indexLanguage, action, value, newValue)
    }
  }
}

function isArray (val) {
  return Object.prototype.toString.call(val) === '[object Array]'
}

function isPlainObject (val) {
  return Object.prototype.toString.call(val) === '[object Object]'
}

function sortAsc (un) {
  let or = {}
  if (isArray(un)) {
    // Sort or don't sort arrays
    if (document.getElementById('noarray').checked) {
      or = un
    } else {
      or = un.sortAsc()
    }
    or.forEach(function (v, i) {
      or[i] = sortAsc(v)
    })
  } else if (isPlainObject(un)) {
    or = {}
    Object.keys(un).sort(function (a, b) {
      if (a.toLowerCase() < b.toLowerCase()) { return -1 }
      if (a.toLowerCase() > b.toLowerCase()) { return 1 }
      return 0
    }).forEach(function (key) {
      or[key] = sortJSON(un[key])
    })
  } else {
    or = un
  }
  return or
}

function sortJSON (json) {
  try {
    let r = sortAsc(json)
    return JSON.parse(JSON.stringify(r, null, 4))
  } catch (ex) {
    console.log('Incorrect JSON object')
    return json
  }
}

function countTranslations (obj) {
  let item
  if (obj instanceof Object) {
    for (item in obj) {
      if (obj.hasOwnProperty(item)) {
        countTranslations(obj[item])
      } else {
        break
      }
    }
  } else {
    nbEntities++
  }
}

api.get(pathApi + '/list-languages', function (req, res) {
  fs.readdir(pathJsonFile, function (err, files) {
    if (err) { throw err }
    let languages = { listLanguages: [] }
    files.forEach(function (file) {
      languages.listLanguages.push({ code: file.substring(0, 2), nbTranslations: 0 })
    })
    res.send(languages)
  })
})

api.get(pathApi + '/count-entities-list-languages', function (req, res) {
  let languages = { listLanguages: [] }
  fs.readdir(pathJsonFile, function (err, files) {
    if (err) { throw err }
    for (const [iterator, file] of files.entries()) {
      jsonfile.readFile(pathJsonFile + file, function (err, obj) {
        if (err) { console.log('Error on read json file : ' + file, 'err', err) }
        nbEntities = 0
        countTranslations(obj)
        languages.listLanguages.push({ code: file.substring(0, 2), nbTranslations: nbEntities })
        if (iterator === files.length - 1) {
          res.send(languages)
        }
      })
    }
  })
})

api.get(pathApi + '/create/:language', function (req, res) {
  let file = pathJsonFile + req.params.language + '.json'
  let obj = {}

  jsonfile.writeFile(file, obj, function (err) {
    if (err) { return console.log('Error on create json file', err) }
    res.sendStatus(200)
  })
})

api.get(pathApi + '/delete/:language', function (req, res) {
  fs.stat(pathJsonFile + req.params.language + '.json', function (err) {
    if (err) { return console.error(err) }

    fs.unlink(pathJsonFile + req.params.language + '.json', function (err) {
      if (err) { return console.log(err) }
      console.log('file "' + req.params.language + '.json" deleted successfully')
      res.sendStatus(200)
    })
  })
})

api.get(pathApi + '/open/:language', function (req, res) {
  res.sendFile(pathJsonFile + req.params.language + '.json')
})

api.get(pathApi + '/download/:language', function (req, res) {
  res.setHeader('Content-Type', 'apilication/json')
  res.download(pathJsonFile + req.params.language + '.json')
})

api.post(pathApi + '/group/:action', function (req, res) {
  const action = req.params.action
  const languages = req.body.languages

  let files = []
  languages.map((file, index) => {
    files[index] = pathJsonFile + languages[index] + '.json'
  })

  const levelsIsDefined = isDefined(req.body.levels)
  const levels = levelsIsDefined ? req.body.levels.split('/') : undefined
  let groupName = req.body.groupName
  let originalGroupName = req.body.originalGroupName
  let i = 0

  files.map((file, index) => {
    jsonfile.readFile(file, function (err, obj) {
      if (err) { console.log('Error on read json file : ' + file, 'err', err) }
      switch (action) {
        case ADD:
          if (levelsIsDefined) {
            targetLevelForAction(obj, levels, i, index, action, groupName)
          } else {
            obj[groupName] = {}
          }
          break
        case UPD:
          if (levelsIsDefined) {
            targetLevelForAction(obj, levels, i, index, action, originalGroupName, groupName)
          } else {
            let contentOfGroup = obj[originalGroupName]
            delete obj[originalGroupName]
            obj[groupName] = contentOfGroup
          }
          break
        case DEL:
          if (levelsIsDefined) {
            targetLevelForAction(obj, levels, i, index, action, groupName)
          } else {
            delete obj[groupName]
          }
          break
      }

      obj = sortJSON(obj)

      jsonfile.writeFile(file, obj, function (err) {
        if (err) { return console.log('Error on ' + action + ' group name on json file : ' + file, 'err', err) }
      })
    })
  })
  res.sendStatus(200)
})

api.post(pathApi + '/translation/:action', function (req, res) {
  const action = req.params.action
  const languages = req.body.languages

  let files = []
  languages.map((file, index) => {
    files[index] = pathJsonFile + languages[index] + '.json'
  })

  const levelsIsDefined = isDefined(req.body.levels)
  const levels = levelsIsDefined ? req.body.levels.split('/') : undefined
  let translation = req.body.translation
  let i = 0

  files.map((file, index) => {
    jsonfile.readFile(file, function (err, obj) {
      if (err) { console.log('Error on read json file : ' + file, 'err', err) }
      switch (action) {
        case ADD:
          if (levelsIsDefined) {
            targetLevelForAction(obj, levels, i, index, action, translation)
          } else {
            obj[translation.key] = translation.value[index]
          }
          break
        case UPD:
          if (levelsIsDefined) {
            targetLevelForAction(obj, levels, i, index, action, translation)
          } else {
            if (translation.originalKey === translation.key) {
              obj[translation.key] = translation.value[index]
            } else {
              delete obj[translation.originalKey]
              obj[translation.key] = translation.value[index]
            }
          }
          break
        case DEL:
          if (levelsIsDefined) {
            targetLevelForAction(obj, levels, i, index, action, translation)
          } else {
            delete obj[translation.key]
          }
          break
      }

      obj = sortJSON(obj)

      jsonfile.writeFile(file, obj, function (err) {
        if (err) { return console.log('Error on ' + action + ' trad on json file : ' + file, 'err', err) }
      })
    })
  })
  res.sendStatus(200)
})

const server = api.listen(7777, 'localhost', function () {
  console.log('API listen on ' + server.address().address + ':' + server.address().port + ' !')
  createFolderIsNotExist('/json/')
})

const electron = require('electron')
// Module to control application life.
const app = electron.app
// Module to create native browser window.
const BrowserWindow = electron.BrowserWindow
const Tray = electron.Tray

// Path is declared in api.js and not here. The api.js file is concatenated at the beginning of this file when building the application.
// const path = require('path')
const url = require('url')

// Keep a global reference of the window object, if you don't, the window will
// be closed automatically when the JavaScript object is garbage collected.
let mainWindow
let appIcon

function createWindow () {
  appIcon = new Tray(path.join(__dirname, '/icons/icon-x64.png'));

  // Create the browser window.
  mainWindow = new BrowserWindow({
    minWidth: 800,
    minHeight: 600,
    width: 1366,
    height: 768,
    icon: path.join(__dirname, '/icons/icon-x64.png'),
    titleBarStyle: 'hidden'
  })

  // and load the index.html of the app.
  mainWindow.loadURL(url.format({
    pathname: path.join(__dirname, 'index.html'),
    protocol: 'file:',
    slashes: true
  }))

  // Open the DevTools.
  // mainWindow.webContents.openDevTools()

  // Emitted when the window is closed.
  mainWindow.on('closed', function () {
    // Dereference the window object, usually you would store windows
    // in an array if your app supports multi windows, this is the time
    // when you should delete the corresponding element.
    mainWindow = null
  })
}

app.requestSingleInstanceLock()

// This method will be called when Electron has finished
// initialization and is ready to create browser windows.
// Some APIs can only be used after this event occurs.
app.whenReady().then(createWindow)

// Quit when all windows are closed.
app.on('window-all-closed', function () {
  // On OS X it is common for applications and their menu bar
  // to stay active until the user quits explicitly with Cmd + Q
  if (process.platform !== 'darwin') {
    app.quit()
  }
})

app.on('activate', function () {
  // On OS X it's common to re-create a window in the app when the
  // dock icon is clicked and there are no other windows open.
  if (mainWindow === null) {
    createWindow()
  }
})

// In this file you can include the rest of your app's specific main process
// code. You can also put them in separate files and require them here.
